"""GUI of QuPath module."""

import platform
from multiprocessing import Manager
from pathlib import Path

import humanize

from aignostics.gui import frame
from aignostics.utils import BasePageBuilder, get_logger

from ._service import InstallProgress, InstallProgressState, Service

logger = get_logger(__name__)


class PageBuilder(BasePageBuilder):
    @staticmethod
    def register_pages() -> None:  # noqa: C901, PLR0915
        from nicegui import app, run, ui  # noq  # noqa: PLC0415

        app.add_static_files("/qupath_assets", Path(__file__).parent / "assets")

        @ui.page("/qupath")
        async def page_index() -> None:  # noqa: C901, PLR0915
            """QuPath Extension."""
            with frame("QuPath Extension", left_sidebar=False):
                # Nothing to do here, just to show the page
                pass

            if platform.system() == "Linux" and platform.machine() in {"aarch64", "arm64"}:
                ui.markdown(
                    """
                    ### Manage your QuPath Extension

                    QuPath is not supported on ARM64 Linux.
                """
                )
                return

            async def install_qupath() -> None:
                def update_install_progress() -> None:
                    """Update the progress indicator with values from the queue."""
                    while not progress_queue.empty():
                        progress: InstallProgress = progress_queue.get()
                        if progress.status is InstallProgressState.DOWNLOADING:
                            if progress.archive_path and progress.archive_size:
                                install_info.set_text(
                                    f"Downloading QuPath v{progress.archive_version} "
                                    f"({humanize.naturalsize(float(progress.archive_size))}) "
                                    f"to {progress.archive_path}"
                                )
                            download_progress.set_value(progress.archive_download_progress_normalized)

                ui.notify("Installing QuPath  ...", type="info")

                progress_queue = Manager().Queue()
                ui.timer(0.1, update_install_progress)

                install_button.props(add="loading")
                install_info.set_text("Connecting with GitHub ...")
                download_progress.set_visibility(True)

                try:
                    app_dir = await run.io_bound(
                        Service.install_qupath,
                        progress_queue=progress_queue,
                    )
                    ui.notify(f"QuPath installed successfully to '{app_dir!s}'.", type="positive")
                except Exception as e:
                    message = f"Failed to install QuPath: {e!s}."
                    logger.exception(message)
                    ui.notify("Failed to install QuPath.", type="negative")

                download_progress.set_visibility(False)
                install_button.props(remove="loading")

                ui.navigate.reload()

            async def uninstall_qupath() -> None:
                uninstall_button.props(add="loading")
                try:
                    await run.io_bound(Service.uninstall_qupath)
                    ui.notify("QuPath uinstalled successfully.", type="positive")
                except Exception as e:
                    message = f"Failed to uninstall QuPath: {e!s}."
                    logger.exception(message)
                    ui.notify("Failed to uninstall QuPath.", type="negative")

                uninstall_button.props(remove="loading")

                ui.navigate.reload()

            async def launch_qupath() -> None:
                """Launch QuPath."""
                launch_button.props(add="loading")

                try:
                    pid = await run.cpu_bound(Service.execute_qupath)
                    if pid:
                        message = f"QuPath launched successfully with process id '{pid}'."
                        logger.info(message)
                        ui.notify(message, type="positive")
                    else:
                        message = "Failed to launch QuPath."
                        logger.error(message)
                        ui.notify(message, type="negative")
                except Exception as e:
                    message = f"Failed to launch QuPath: {e!s}."
                    logger.exception(message)
                    ui.notify("Failed to launch QuPath.", type="negative")

                launch_button.props(remove="loading")

            spinner = ui.spinner(size="lg").classes(
                "absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 z-10"
            )
            version = await run.io_bound(Service.get_version)
            spinner.delete()
            expected_version = Service.get_expected_version()

            ui.markdown(
                """
                    ### Manage your QuPath Extension
                """
            )
            with ui.row().classes("w-full justify-start"):
                with ui.column().classes("w-2/5"):
                    with ui.card().classes("w-full"):
                        if version:
                            if version.version == expected_version:
                                install_info = ui.label(
                                    f"QuPath {expected_version} is installed and ready to execute. "
                                    "Go to a completed application result and click the QuPath button, "
                                    "or open directly from here."
                                )
                            else:
                                install_info = ui.label(
                                    f"QuPath {version.version} is installed, but version {expected_version} "
                                    "is expected. Please update to the latest version by reinstalling."
                                )
                        else:
                            install_info = ui.label(
                                "Install QuPath to enable visualizing your Whole Slide Image and application results "
                                "with one click. "
                                f"QuPath will be installed at '{Service.get_installation_path()}'. "
                            )

                        download_progress = ui.linear_progress(value=0, show_value=False).props("instant-feedback")
                        download_progress.set_visibility(False)

                        with ui.row().classes("w-full justify-between items-center"):
                            launch_button = ui.button(
                                "Open",
                                on_click=launch_qupath,
                                icon="visibility",
                            ).mark("BUTTON_QUPATH_LAUNCH")
                            if not version:
                                launch_button.disable()
                            ui.space()
                            install_button = ui.button(
                                "Install" if not version else "Reinstall",
                                on_click=install_qupath,
                                icon="install_desktop",
                            ).mark("BUTTON_QUPATH_INSTALL")
                            uninstall_button = ui.button(
                                "Uninstall",
                                on_click=uninstall_qupath,
                                icon="extension_off",
                            ).mark("BUTTON_QUPATH_INSTALL")
                            if not version:
                                uninstall_button.disable()

                    ui.markdown(
                        """
                            ###### What is QuPath?
                            QuPath [1, 2] is a powerful open-source software for digital pathology.
                            It allows you to visualize and annotate whole slide images (WSIs) with ease.

                            Using the Aignostics Launchpad you can install QuPath with one click, and
                            start visualizing your WSIs and application results with ease.

                            *References:*

                            1. <a href="https://qupath.github.io/" target="_blank">QuPath website</a>

                            2. Bankhead, P. et al. QuPath:
                                <a href="https://doi.org/10.1038/s41598-017-17204-5" target="_blank">
                                Open source software for digital pathology image analysis.
                                Scientific Reports (2017)</a>

                            3. <a href="https://qupath.readthedocs.io/en/stable/docs/intro/acknowledgements.html"
                                target="_blank">License</a>
                        """
                    )
                ui.space()
                with ui.column().classes("w-2/5"), ui.row().classes("w-1/2 justify-center content-center"):
                    ui.space()
                    if version:
                        if version.version == expected_version:
                            animation = "/qupath_assets/microscope.lottie"
                        else:
                            animation = "/qupath_assets/update.lottie"
                    else:
                        animation = "/qupath_assets/download.lottie"
                    ui.html(
                        f"<dotlottie-player "
                        f'src="{animation}" '
                        f'background="transparent" '
                        f'speed="1" '
                        f'style="width: 300px; height: 300px" '
                        f'direction="1" '
                        f'playMode="normal" '
                        f"loop "
                        f"autoplay>"
                        f"</dotlottie-player>"
                    )
                    ui.space()




"""
    Aignostics Platform API reference

     Pagination is done via `page` and `page_size`. Sorting via `sort` query parameter. The `sort` query parameter can be provided multiple times. The sorting direction can be indicated via `+` (ascending) or `-` (descending) (e.g. `/v1/applications?sort=+name)`.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ApplicationRunStatus(str, Enum):
    """
    ApplicationRunStatus
    """

    """
    allowed enum values
    """
    CANCELED_SYSTEM = 'canceled_system'
    CANCELED_USER = 'canceled_user'
    COMPLETED = 'completed'
    COMPLETED_WITH_ERROR = 'completed_with_error'
    RECEIVED = 'received'
    REJECTED = 'rejected'
    RUNNING = 'running'
    SCHEDULED = 'scheduled'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ApplicationRunStatus from a JSON string"""
        return cls(json.loads(json_str))



"""Utils for printing wsi files."""

from pathlib import Path
from typing import Any

import humanize

from aignostics.utils import console


def get_tag_info(tag_str: str) -> str:
    """Convert DICOM tag to human readable name.

    Args:
        tag_str(str): DICOM tag string in format '00100010'.

    Returns:
        str: Human readable name of the DICOM tag or the original tag string if not found.
    """
    from pydicom.datadict import dictionary_description  # noqa: PLC0415

    try:
        # Convert string tag like '00100010' to tuple format (0010,0010)
        tag_tuple = (int(tag_str[0:4], 16), int(tag_str[4:8], 16))
        description = dictionary_description(tag_tuple)
        return f"{description}" if description else tag_str
    except KeyError:
        return tag_str


def print_file_info(file_info: dict[str, Any], indent: int = 0) -> None:  # noqa: C901, PLR0912, PLR0915
    """Print formatted file information.

    Args:
        file_info (dict): Dictionary containing file information.
        indent (int): Indentation level for formatting.
    """
    prefix = "  " * indent

    # Keep existing basic info
    console.print(f"{prefix}[key]Path:[/key] {Path(file_info['path']).name}")
    console.print(f"{prefix}[key]Size:[/key] {humanize.naturalsize(file_info['size'])}")
    console.print(f"{prefix}[key]Type:[/key] {file_info['type']}")

    if "instance_uid" in file_info:
        console.print(f"{prefix}[key]Instance UID:[/key] {file_info['instance_uid']}")

    console.print(f"{prefix}[key]Frame of reference UID:[/key] {file_info['frame_of_reference_uid']}")

    # Add WSI/pyramidal specific information

    if file_info.get("is_pyramidal"):
        console.print(f"{prefix}[key]Image Type:[/key] Pyramidal WSI")
        console.print(f"{prefix}[key]Number of Frames:[/key] {file_info['num_frames']}")
        if file_info.get("optical_paths"):
            console.print(f"{prefix}[key]Optical Paths:[/key] {file_info['optical_paths']}")
        if file_info.get("focal_planes"):
            console.print(f"{prefix}[key]Focal Planes:[/key] {file_info['focal_planes']}")
        if file_info.get("total_pixel_matrix"):
            matrix = file_info["total_pixel_matrix"]
            console.print(f"{prefix}[key]Total Pixel Matrix:[/key] {matrix[0]} x {matrix[1]}")

    if file_info.get("modality") == "ANN":
        console.print(f"{prefix}[key]Coordinate type:[/key] {file_info['coordinate_type']}")
        console.print(f"{prefix}[key]Annotation Groups:[/key] {len(file_info['annotation_groups'])}")
        prefix = "  " + prefix
        for group in file_info["annotation_groups"]:
            console.print("")
            console.print(f"{prefix}[key]UUID:[/key] {group['uid']}")
            console.print(f"{prefix}[key]Label:[/key] {group['label']}")
            console.print(f"{prefix}[key]Property type:[/key] {group['property_type']}")
            console.print(f"{prefix}[key]Graphic type:[/key] {group['graphic_type']}")
            console.print(f"{prefix}[key]Count:[/key] {group['count']}")
            col_range = f"[{group['col_min']:.1f}, {group['col_max']:.1f}]"
            row_range = f"[{group['row_min']:.1f}, {group['row_max']:.1f}]"
            console.print(f"{prefix}[key]Column [Min,Max]:[/key] {col_range}")
            console.print(f"{prefix}[key]Row [Min,Max]:[/key] {row_range}")
            console.print(f"{prefix}[key]First:[/key] {group['first']}")
        console.print("")

    prefix = "  " * indent

    # Keep existing image-specific information
    if "dimensions" in file_info:
        if isinstance(file_info["dimensions"], tuple):
            dim_str = f"{file_info['dimensions'][0]} x {file_info['dimensions'][1]}"
        else:
            dim_str = str(file_info["dimensions"])
        console.print(f"{prefix}[key]Dimensions:[/key] {dim_str}")

        if "photometric_interpretation" in file_info:
            console.print(f"{prefix}[key]Color Space:[/key] {file_info['photometric_interpretation']}")

        if "bits_allocated" in file_info and "bits_stored" in file_info:
            bits_str = f"{file_info['bits_allocated']} allocated, {file_info['bits_stored']} stored"
            console.print(f"{prefix}[key]Bits:[/key] {bits_str}")

        if "samples_per_pixel" in file_info:
            console.print(f"{prefix}[key]Samples per Pixel:[/key] {file_info['samples_per_pixel']}")

        if "image_type" in file_info:
            console.print(f"{prefix}[key]DICOM Image Type:[/key] {' / '.join(file_info['image_type'])}")

    if file_info.get("pyramid_info"):
        console.print(f"\n{prefix}[key]Pyramid Structure:[/key]")
        for level in file_info["pyramid_info"]:
            frame_size = f"{level['frame_size'][0]} x {level['frame_size'][1]}"
            console.print(f"{prefix}  Level {level['level']}: {level['frame_count']} frames @ {frame_size} pixels")


def print_series_info(series_data: dict[str, Any], indent: int = 0) -> None:
    """Print formatted series information."""
    prefix = "  " * indent
    console.print(f"{prefix}[key]Files:[/key] {series_data['file_count']}")
    console.print(f"{prefix}[key]Modality:[/key] {series_data['modality']}")


def format_dicom_time(time_str: str) -> str:
    """Format DICOM time string to HH:MM:SS.

    Args:
        time_str (str): DICOM time string in the format HHMMSS.

    Returns:
        str: Formatted time string in the format HH:MM:SS or the original string if invalid.
    """
    if not time_str or len(time_str) < 6:  # noqa: PLR2004
        return time_str
    return f"{time_str[0:2]}:{time_str[2:4]}:{time_str[4:6]}"


def print_study_info(study_data: dict[str, Any], indent: int = 0) -> None:
    """Print formatted study information.

    Args:
        study_data (dict): Dictionary containing study information.
        indent (int): Indentation level for formatting.
    """
    prefix = "  " * indent

    # Patient Section
    console.print(f"{prefix} [header]('Patient:')[/header]")
    console.print(f"{prefix}  [key]ID:[/key] {study_data['patient_info']['id']}")
    console.print(f"{prefix}  [key]Name:[/key] {study_data['patient_info']['name']}")
    console.print(f"{prefix}  [key]Gender:[/key] {study_data['patient_info']['gender']}")
    console.print(f"{prefix}  [key]Birth Date:[/key] {study_data['patient_info']['birth_date']}")

    # Study Section
    console.print(f"\n{prefix}[header]('Study:')[/header]")
    console.print(f"{prefix}  [key]Accession #:[/key] {study_data['study_info']['accession_number']}")
    console.print(f"{prefix}  [key]ID:[/key] {study_data['study_info']['study_id']}")
    console.print(f"{prefix}  [key]Date:[/key] {study_data['study_info']['study_date']}")
    console.print(f"{prefix}  [key]Time:[/key] {format_dicom_time(study_data['study_info']['study_time'])}")
    console.print(f"{prefix}  [key]UID:[/key] {study_data['study_info']['study_uid']}")

    # Clinical Trial Section
    if any(study_data["clinical_trial"].values()):  # Only show if any values exist
        console.print(f"\n{prefix}[header]('Clinical Trial:')[/header]")
        console.print(f"{prefix}  [key]Sponsor:[/key] {study_data['clinical_trial']['sponsor_name']}")
        console.print(f"{prefix}  [key]Protocol ID:[/key] {study_data['clinical_trial']['protocol_id']}")
        console.print(f"{prefix}  [key]Protocol Name:[/key] {study_data['clinical_trial']['protocol_name']}")
        console.print(f"{prefix}  [key]Site Name:[/key] {study_data['clinical_trial']['site_name']}")


def print_slide_info(slide_data: dict[str, Any], indent: int = 0, verbose: bool = False) -> None:
    """Print formatted slide (container) information.

    Args:
        slide_data (dict): Dictionary containing slide information.
        indent (int): Indentation level for formatting.
        verbose (bool): If True, print detailed file information.
    """
    prefix = "  " * indent

    # Specimen Section
    console.print(f"{prefix}[header]('Specimen:')[/header]")
    console.print(f"{prefix}  [key]Description:[/key] {slide_data['specimen_info']['description']}")
    console.print(f"{prefix}  [key]Anatomical Structure:[/key] {slide_data['specimen_info']['anatomical_structure']}")
    console.print(f"{prefix}  [key]Collection Method:[/key] {slide_data['specimen_info']['collection_method']}")
    if slide_data["specimen_info"]["parent_specimens"]:
        console.print(
            f"{prefix}  [key]Parent Specimens:[/key] {', '.join(slide_data['specimen_info']['parent_specimens'])}"
        )
    console.print(f"{prefix}  [key]Embedding Medium:[/key] {slide_data['specimen_info']['embedding_medium']}")

    # Equipment Section
    console.print(f"\n{prefix}[header]('Equipment:')[/header]")
    console.print(f"{prefix}  [key]Manufacturer:[/key] {slide_data['equipment_info']['manufacturer']}")
    console.print(f"{prefix}  [key]Model Name:[/key] {slide_data['equipment_info']['model_name']}")
    console.print(f"{prefix}  [key]Serial Number:[/key] {slide_data['equipment_info']['device_serial_number']}")
    console.print(f"{prefix}  [key]Software Version:[/key] {slide_data['equipment_info']['software_version']}")
    console.print(f"{prefix}  [key]Institution:[/key] {slide_data['equipment_info']['institution_name']}")

    # Series Section
    console.print(f"\n{prefix}[header]('Series:')[/header]")
    total_files = sum(len(series["files"]) for series in slide_data["series"].values())
    console.print(f"{prefix}  [key]Total Files:[/key] {total_files}")
    console.print(f"{prefix}  [key]Series Count:[/key] {len(slide_data['series'])}")

    # Individual Series with their files
    for series_uid, series_data in slide_data["series"].items():
        console.print(f"\n{prefix}  [key]Series UID:[/key] {series_uid}")
        console.print(f"{prefix}    [key]Description:[/key] {series_data['description']}")
        console.print(f"{prefix}    [key]Modality:[/key] {series_data['modality']}")
        console.print(f"{prefix}    [key]Files:[/key] {len(series_data['files'])}")

        # Print file details if verbose
        if verbose:
            for file_info in series_data["files"]:
                console.print()
                print_file_info(file_info, indent=indent + 3)




"""
    Aignostics Platform API reference

     Pagination is done via `page` and `page_size`. Sorting via `sort` query parameter. The `sort` query parameter can be provided multiple times. The sorting direction can be indicated via `+` (ascending) or `-` (descending) (e.g. `/v1/applications?sort=+name)`.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictStr
from typing import Any, List, Optional
from typing_extensions import Annotated
from aignx.codegen.models.application_read_response import ApplicationReadResponse
from aignx.codegen.models.application_version_read_response import ApplicationVersionReadResponse
from aignx.codegen.models.item_result_read_response import ItemResultReadResponse
from aignx.codegen.models.item_status import ItemStatus
from aignx.codegen.models.run_creation_request import RunCreationRequest
from aignx.codegen.models.run_creation_response import RunCreationResponse
from aignx.codegen.models.run_read_response import RunReadResponse

from aignx.codegen.api_client import ApiClient, RequestSerialized
from aignx.codegen.api_response import ApiResponse
from aignx.codegen.rest import RESTResponseType


class PublicApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def cancel_application_run_v1_runs_application_run_id_cancel_post(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> object:
        """Cancel Application Run

        The application run can be canceled by the user who created the application run.  The execution can be canceled any time while the application is not in a final state. The pending items will not be processed and will not add to the cost.  When the application is canceled, the already completed items stay available for download.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._cancel_application_run_v1_runs_application_run_id_cancel_post_serialize(
            application_run_id=application_run_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "object",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def cancel_application_run_v1_runs_application_run_id_cancel_post_with_http_info(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[object]:
        """Cancel Application Run

        The application run can be canceled by the user who created the application run.  The execution can be canceled any time while the application is not in a final state. The pending items will not be processed and will not add to the cost.  When the application is canceled, the already completed items stay available for download.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._cancel_application_run_v1_runs_application_run_id_cancel_post_serialize(
            application_run_id=application_run_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "object",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def cancel_application_run_v1_runs_application_run_id_cancel_post_without_preload_content(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Cancel Application Run

        The application run can be canceled by the user who created the application run.  The execution can be canceled any time while the application is not in a final state. The pending items will not be processed and will not add to the cost.  When the application is canceled, the already completed items stay available for download.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._cancel_application_run_v1_runs_application_run_id_cancel_post_serialize(
            application_run_id=application_run_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "object",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _cancel_application_run_v1_runs_application_run_id_cancel_post_serialize(
        self,
        application_run_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if application_run_id is not None:
            _path_params['application_run_id'] = application_run_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2AuthorizationCodeBearer'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v1/runs/{application_run_id}/cancel',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def create_application_run_v1_runs_post(
        self,
        run_creation_request: RunCreationRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RunCreationResponse:
        """Create Application Run

        The endpoint is used to process the input items by the chosen application version. The endpoint returns the `application_run_id`. The processing fo the items is done asynchronously.  To check the status or cancel the execution, use the /v1/runs/{application_run_id} endpoint.  ### Payload  The payload includes `application_version_id` and `items` base fields.  `application_version_id` is the id used for `/v1/versions/{application_id}` endpoint.  `items` includes the list of the items to process (slides, in case of HETA application). Every item has a set of standard fields defined by the API, plus the metadata, specific to the chosen application.  Example payload structure with the comments: ``` {     application_version_id: \"heta:v1.0.0\",     items: [{         \"reference\": \"slide_1\"  <-- Input ID to connect the input and the output artifact         \"input_artifacts\": [{             \"name\": \"input_slide\"  <-- Name of the artifact defined by the application (For HETA it is\"input_slide\")             \"download_url\": \"https://...\" <-- signed URL to the input file in the S3 or GCS. Should be valid for more than 6 days             \"metadata\": { <-- The metadata fields defined by the application. (The example fields set for a slide files are provided)                 \"checksum_crc32c\": \"abc12==\",                 \"mime_type\": \"image/tiff\",                 \"height\": 100,                 \"weight\": 500,                 \"mpp\": 0.543             }         }]     }] } ```  ### Response  The endpoint returns the application run UUID. After that the job is scheduled for the execution in the background.  To check the status of the run call `v1/runs/{application_run_id}`.  ### Rejection  Apart from the authentication, authorization and malformed input error, the request can be rejected when the quota limit is exceeded. More details on quotas is described in the documentation

        :param run_creation_request: (required)
        :type run_creation_request: RunCreationRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_application_run_v1_runs_post_serialize(
            run_creation_request=run_creation_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "RunCreationResponse",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_application_run_v1_runs_post_with_http_info(
        self,
        run_creation_request: RunCreationRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RunCreationResponse]:
        """Create Application Run

        The endpoint is used to process the input items by the chosen application version. The endpoint returns the `application_run_id`. The processing fo the items is done asynchronously.  To check the status or cancel the execution, use the /v1/runs/{application_run_id} endpoint.  ### Payload  The payload includes `application_version_id` and `items` base fields.  `application_version_id` is the id used for `/v1/versions/{application_id}` endpoint.  `items` includes the list of the items to process (slides, in case of HETA application). Every item has a set of standard fields defined by the API, plus the metadata, specific to the chosen application.  Example payload structure with the comments: ``` {     application_version_id: \"heta:v1.0.0\",     items: [{         \"reference\": \"slide_1\"  <-- Input ID to connect the input and the output artifact         \"input_artifacts\": [{             \"name\": \"input_slide\"  <-- Name of the artifact defined by the application (For HETA it is\"input_slide\")             \"download_url\": \"https://...\" <-- signed URL to the input file in the S3 or GCS. Should be valid for more than 6 days             \"metadata\": { <-- The metadata fields defined by the application. (The example fields set for a slide files are provided)                 \"checksum_crc32c\": \"abc12==\",                 \"mime_type\": \"image/tiff\",                 \"height\": 100,                 \"weight\": 500,                 \"mpp\": 0.543             }         }]     }] } ```  ### Response  The endpoint returns the application run UUID. After that the job is scheduled for the execution in the background.  To check the status of the run call `v1/runs/{application_run_id}`.  ### Rejection  Apart from the authentication, authorization and malformed input error, the request can be rejected when the quota limit is exceeded. More details on quotas is described in the documentation

        :param run_creation_request: (required)
        :type run_creation_request: RunCreationRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_application_run_v1_runs_post_serialize(
            run_creation_request=run_creation_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "RunCreationResponse",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_application_run_v1_runs_post_without_preload_content(
        self,
        run_creation_request: RunCreationRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create Application Run

        The endpoint is used to process the input items by the chosen application version. The endpoint returns the `application_run_id`. The processing fo the items is done asynchronously.  To check the status or cancel the execution, use the /v1/runs/{application_run_id} endpoint.  ### Payload  The payload includes `application_version_id` and `items` base fields.  `application_version_id` is the id used for `/v1/versions/{application_id}` endpoint.  `items` includes the list of the items to process (slides, in case of HETA application). Every item has a set of standard fields defined by the API, plus the metadata, specific to the chosen application.  Example payload structure with the comments: ``` {     application_version_id: \"heta:v1.0.0\",     items: [{         \"reference\": \"slide_1\"  <-- Input ID to connect the input and the output artifact         \"input_artifacts\": [{             \"name\": \"input_slide\"  <-- Name of the artifact defined by the application (For HETA it is\"input_slide\")             \"download_url\": \"https://...\" <-- signed URL to the input file in the S3 or GCS. Should be valid for more than 6 days             \"metadata\": { <-- The metadata fields defined by the application. (The example fields set for a slide files are provided)                 \"checksum_crc32c\": \"abc12==\",                 \"mime_type\": \"image/tiff\",                 \"height\": 100,                 \"weight\": 500,                 \"mpp\": 0.543             }         }]     }] } ```  ### Response  The endpoint returns the application run UUID. After that the job is scheduled for the execution in the background.  To check the status of the run call `v1/runs/{application_run_id}`.  ### Rejection  Apart from the authentication, authorization and malformed input error, the request can be rejected when the quota limit is exceeded. More details on quotas is described in the documentation

        :param run_creation_request: (required)
        :type run_creation_request: RunCreationRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_application_run_v1_runs_post_serialize(
            run_creation_request=run_creation_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "RunCreationResponse",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_application_run_v1_runs_post_serialize(
        self,
        run_creation_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if run_creation_request is not None:
            _body_params = run_creation_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2AuthorizationCodeBearer'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v1/runs',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_application_run_results_v1_runs_application_run_id_results_delete(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete Application Run Results

        Delete the application run results. It can only be called when the application is in a final state (meaning it's not in `received` or `pending` states). To delete the results of the running artifacts, first call `POST /v1/runs/{application_run_id}/cancel` to cancel the application run.  The output results are deleted automatically 30 days after the application run is finished.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_run_results_v1_runs_application_run_id_results_delete_serialize(
            application_run_id=application_run_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_application_run_results_v1_runs_application_run_id_results_delete_with_http_info(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete Application Run Results

        Delete the application run results. It can only be called when the application is in a final state (meaning it's not in `received` or `pending` states). To delete the results of the running artifacts, first call `POST /v1/runs/{application_run_id}/cancel` to cancel the application run.  The output results are deleted automatically 30 days after the application run is finished.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_run_results_v1_runs_application_run_id_results_delete_serialize(
            application_run_id=application_run_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_application_run_results_v1_runs_application_run_id_results_delete_without_preload_content(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete Application Run Results

        Delete the application run results. It can only be called when the application is in a final state (meaning it's not in `received` or `pending` states). To delete the results of the running artifacts, first call `POST /v1/runs/{application_run_id}/cancel` to cancel the application run.  The output results are deleted automatically 30 days after the application run is finished.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_application_run_results_v1_runs_application_run_id_results_delete_serialize(
            application_run_id=application_run_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_application_run_results_v1_runs_application_run_id_results_delete_serialize(
        self,
        application_run_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if application_run_id is not None:
            _path_params['application_run_id'] = application_run_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2AuthorizationCodeBearer'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/v1/runs/{application_run_id}/results',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_run_v1_runs_application_run_id_get(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        include: Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RunReadResponse:
        """Get Run

        Returns the details of the application run. The application run is available as soon as it is created via `POST /runs/` endpoint. To download the items results, call `/runs/{application_run_id}/results`.  The application is only available to the user who triggered it, regardless of the role.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param include:
        :type include: List[object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_run_v1_runs_application_run_id_get_serialize(
            application_run_id=application_run_id,
            include=include,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RunReadResponse",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_run_v1_runs_application_run_id_get_with_http_info(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        include: Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RunReadResponse]:
        """Get Run

        Returns the details of the application run. The application run is available as soon as it is created via `POST /runs/` endpoint. To download the items results, call `/runs/{application_run_id}/results`.  The application is only available to the user who triggered it, regardless of the role.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param include:
        :type include: List[object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_run_v1_runs_application_run_id_get_serialize(
            application_run_id=application_run_id,
            include=include,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RunReadResponse",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_run_v1_runs_application_run_id_get_without_preload_content(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        include: Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Run

        Returns the details of the application run. The application run is available as soon as it is created via `POST /runs/` endpoint. To download the items results, call `/runs/{application_run_id}/results`.  The application is only available to the user who triggered it, regardless of the role.

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param include:
        :type include: List[object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_run_v1_runs_application_run_id_get_serialize(
            application_run_id=application_run_id,
            include=include,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RunReadResponse",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_run_v1_runs_application_run_id_get_serialize(
        self,
        application_run_id,
        include,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'include': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if application_run_id is not None:
            _path_params['application_run_id'] = application_run_id
        # process the query parameters
        if include is not None:
            
            _query_params.append(('include', include))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2AuthorizationCodeBearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/runs/{application_run_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_application_runs_v1_runs_get(
        self,
        application_id: Annotated[Optional[StrictStr], Field(description="Optional application ID filter")] = None,
        application_version: Annotated[Optional[StrictStr], Field(description="Optional application version filter")] = None,
        include: Annotated[Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]], Field(description="Request optional output values. Used internally by the platform")] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[RunReadResponse]:
        """List Application Runs

        The endpoint returns the application runs triggered by the caller. After the application run is created by POST /v1/runs, it becomes available for the current endpoint

        :param application_id: Optional application ID filter
        :type application_id: str
        :param application_version: Optional application version filter
        :type application_version: str
        :param include: Request optional output values. Used internally by the platform
        :type include: List[object]
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_application_runs_v1_runs_get_serialize(
            application_id=application_id,
            application_version=application_version,
            include=include,
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RunReadResponse]",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_application_runs_v1_runs_get_with_http_info(
        self,
        application_id: Annotated[Optional[StrictStr], Field(description="Optional application ID filter")] = None,
        application_version: Annotated[Optional[StrictStr], Field(description="Optional application version filter")] = None,
        include: Annotated[Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]], Field(description="Request optional output values. Used internally by the platform")] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[RunReadResponse]]:
        """List Application Runs

        The endpoint returns the application runs triggered by the caller. After the application run is created by POST /v1/runs, it becomes available for the current endpoint

        :param application_id: Optional application ID filter
        :type application_id: str
        :param application_version: Optional application version filter
        :type application_version: str
        :param include: Request optional output values. Used internally by the platform
        :type include: List[object]
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_application_runs_v1_runs_get_serialize(
            application_id=application_id,
            application_version=application_version,
            include=include,
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RunReadResponse]",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_application_runs_v1_runs_get_without_preload_content(
        self,
        application_id: Annotated[Optional[StrictStr], Field(description="Optional application ID filter")] = None,
        application_version: Annotated[Optional[StrictStr], Field(description="Optional application version filter")] = None,
        include: Annotated[Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]], Field(description="Request optional output values. Used internally by the platform")] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Application Runs

        The endpoint returns the application runs triggered by the caller. After the application run is created by POST /v1/runs, it becomes available for the current endpoint

        :param application_id: Optional application ID filter
        :type application_id: str
        :param application_version: Optional application version filter
        :type application_version: str
        :param include: Request optional output values. Used internally by the platform
        :type include: List[object]
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_application_runs_v1_runs_get_serialize(
            application_id=application_id,
            application_version=application_version,
            include=include,
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RunReadResponse]",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_application_runs_v1_runs_get_serialize(
        self,
        application_id,
        application_version,
        include,
        page,
        page_size,
        sort,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'include': 'multi',
            'sort': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if application_id is not None:
            
            _query_params.append(('application_id', application_id))
            
        if application_version is not None:
            
            _query_params.append(('application_version', application_version))
            
        if include is not None:
            
            _query_params.append(('include', include))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('page_size', page_size))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2AuthorizationCodeBearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/runs',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_applications_v1_applications_get(
        self,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ApplicationReadResponse]:
        """List Applications

        Returns the list of the applications, available to the caller.  The application is available if any of the version of the application is assigned to the user organization. To switch between organizations, the user should re-login and choose the needed organization.

        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_applications_v1_applications_get_serialize(
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApplicationReadResponse]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_applications_v1_applications_get_with_http_info(
        self,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ApplicationReadResponse]]:
        """List Applications

        Returns the list of the applications, available to the caller.  The application is available if any of the version of the application is assigned to the user organization. To switch between organizations, the user should re-login and choose the needed organization.

        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_applications_v1_applications_get_serialize(
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApplicationReadResponse]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_applications_v1_applications_get_without_preload_content(
        self,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Applications

        Returns the list of the applications, available to the caller.  The application is available if any of the version of the application is assigned to the user organization. To switch between organizations, the user should re-login and choose the needed organization.

        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_applications_v1_applications_get_serialize(
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApplicationReadResponse]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_applications_v1_applications_get_serialize(
        self,
        page,
        page_size,
        sort,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'sort': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('page_size', page_size))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2AuthorizationCodeBearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/applications',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_run_results_v1_runs_application_run_id_results_get(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        item_id__in: Annotated[Optional[List[StrictStr]], Field(description="Filter for items ids")] = None,
        reference__in: Annotated[Optional[List[StrictStr]], Field(description="Filter for items by their reference from the input payload")] = None,
        status__in: Annotated[Optional[List[ItemStatus]], Field(description="Filter for items in certain statuses")] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ItemResultReadResponse]:
        """List Run Results

        Get the list of the results for the run items

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param item_id__in: Filter for items ids
        :type item_id__in: List[str]
        :param reference__in: Filter for items by their reference from the input payload
        :type reference__in: List[str]
        :param status__in: Filter for items in certain statuses
        :type status__in: List[ItemStatus]
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_run_results_v1_runs_application_run_id_results_get_serialize(
            application_run_id=application_run_id,
            item_id__in=item_id__in,
            reference__in=reference__in,
            status__in=status__in,
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ItemResultReadResponse]",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_run_results_v1_runs_application_run_id_results_get_with_http_info(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        item_id__in: Annotated[Optional[List[StrictStr]], Field(description="Filter for items ids")] = None,
        reference__in: Annotated[Optional[List[StrictStr]], Field(description="Filter for items by their reference from the input payload")] = None,
        status__in: Annotated[Optional[List[ItemStatus]], Field(description="Filter for items in certain statuses")] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ItemResultReadResponse]]:
        """List Run Results

        Get the list of the results for the run items

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param item_id__in: Filter for items ids
        :type item_id__in: List[str]
        :param reference__in: Filter for items by their reference from the input payload
        :type reference__in: List[str]
        :param status__in: Filter for items in certain statuses
        :type status__in: List[ItemStatus]
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_run_results_v1_runs_application_run_id_results_get_serialize(
            application_run_id=application_run_id,
            item_id__in=item_id__in,
            reference__in=reference__in,
            status__in=status__in,
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ItemResultReadResponse]",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_run_results_v1_runs_application_run_id_results_get_without_preload_content(
        self,
        application_run_id: Annotated[StrictStr, Field(description="Application run id, returned by `POST /runs/` endpoint")],
        item_id__in: Annotated[Optional[List[StrictStr]], Field(description="Filter for items ids")] = None,
        reference__in: Annotated[Optional[List[StrictStr]], Field(description="Filter for items by their reference from the input payload")] = None,
        status__in: Annotated[Optional[List[ItemStatus]], Field(description="Filter for items in certain statuses")] = None,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Run Results

        Get the list of the results for the run items

        :param application_run_id: Application run id, returned by `POST /runs/` endpoint (required)
        :type application_run_id: str
        :param item_id__in: Filter for items ids
        :type item_id__in: List[str]
        :param reference__in: Filter for items by their reference from the input payload
        :type reference__in: List[str]
        :param status__in: Filter for items in certain statuses
        :type status__in: List[ItemStatus]
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_run_results_v1_runs_application_run_id_results_get_serialize(
            application_run_id=application_run_id,
            item_id__in=item_id__in,
            reference__in=reference__in,
            status__in=status__in,
            page=page,
            page_size=page_size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ItemResultReadResponse]",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_run_results_v1_runs_application_run_id_results_get_serialize(
        self,
        application_run_id,
        item_id__in,
        reference__in,
        status__in,
        page,
        page_size,
        sort,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'item_id__in': 'multi',
            'reference__in': 'multi',
            'status__in': 'multi',
            'sort': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if application_run_id is not None:
            _path_params['application_run_id'] = application_run_id
        # process the query parameters
        if item_id__in is not None:
            
            _query_params.append(('item_id__in', item_id__in))
            
        if reference__in is not None:
            
            _query_params.append(('reference__in', reference__in))
            
        if status__in is not None:
            
            _query_params.append(('status__in', status__in))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('page_size', page_size))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2AuthorizationCodeBearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/runs/{application_run_id}/results',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_versions_by_application_id_v1_applications_application_id_versions_get(
        self,
        application_id: StrictStr,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        version: Optional[StrictStr] = None,
        include: Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ApplicationVersionReadResponse]:
        """List Versions By Application Id

        Returns the list of the application versions for this application, available to the caller.  The application version is available if it is assigned to the user's organization.  The application versions are assigned to the organization by the Aignostics admin. To assign or unassign a version from your organization, please contact Aignostics support team.

        :param application_id: (required)
        :type application_id: str
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param version:
        :type version: str
        :param include:
        :type include: List[object]
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_versions_by_application_id_v1_applications_application_id_versions_get_serialize(
            application_id=application_id,
            page=page,
            page_size=page_size,
            version=version,
            include=include,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApplicationVersionReadResponse]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_versions_by_application_id_v1_applications_application_id_versions_get_with_http_info(
        self,
        application_id: StrictStr,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        version: Optional[StrictStr] = None,
        include: Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ApplicationVersionReadResponse]]:
        """List Versions By Application Id

        Returns the list of the application versions for this application, available to the caller.  The application version is available if it is assigned to the user's organization.  The application versions are assigned to the organization by the Aignostics admin. To assign or unassign a version from your organization, please contact Aignostics support team.

        :param application_id: (required)
        :type application_id: str
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param version:
        :type version: str
        :param include:
        :type include: List[object]
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_versions_by_application_id_v1_applications_application_id_versions_get_serialize(
            application_id=application_id,
            page=page,
            page_size=page_size,
            version=version,
            include=include,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApplicationVersionReadResponse]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_versions_by_application_id_v1_applications_application_id_versions_get_without_preload_content(
        self,
        application_id: StrictStr,
        page: Optional[Annotated[int, Field(strict=True, ge=1)]] = None,
        page_size: Optional[Annotated[int, Field(le=100, strict=True, ge=5)]] = None,
        version: Optional[StrictStr] = None,
        include: Optional[Annotated[List[Any], Field(min_length=1, max_length=1)]] = None,
        sort: Optional[List[StrictStr]] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Versions By Application Id

        Returns the list of the application versions for this application, available to the caller.  The application version is available if it is assigned to the user's organization.  The application versions are assigned to the organization by the Aignostics admin. To assign or unassign a version from your organization, please contact Aignostics support team.

        :param application_id: (required)
        :type application_id: str
        :param page:
        :type page: int
        :param page_size:
        :type page_size: int
        :param version:
        :type version: str
        :param include:
        :type include: List[object]
        :param sort:
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_versions_by_application_id_v1_applications_application_id_versions_get_serialize(
            application_id=application_id,
            page=page,
            page_size=page_size,
            version=version,
            include=include,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[ApplicationVersionReadResponse]",
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_versions_by_application_id_v1_applications_application_id_versions_get_serialize(
        self,
        application_id,
        page,
        page_size,
        version,
        include,
        sort,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'include': 'multi',
            'sort': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if application_id is not None:
            _path_params['application_id'] = application_id
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('page_size', page_size))
            
        if version is not None:
            
            _query_params.append(('version', version))
            
        if include is not None:
            
            _query_params.append(('include', include))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'OAuth2AuthorizationCodeBearer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/applications/{application_id}/versions',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )



"""GUI of bucket module."""

from pathlib import Path

from aignostics.gui import frame

from ..utils import BasePageBuilder  # noqa: TID252
from ._service import Service


class PageBuilder(BasePageBuilder):
    @staticmethod
    def register_pages() -> None:  # noqa: C901, PLR0915
        from nicegui import app, binding, run, ui  # noqa: PLC0415

        app.add_static_files("/bucket_assets", Path(__file__).parent / "assets")

        @binding.bindable_dataclass
        class BucketForm:
            """Bucket form."""

            grid: ui.aggrid | None = None
            delete_button: ui.button | None = None
            spinner: ui.spinner | None = None

        bucket_form = BucketForm()

        @ui.page("/bucket")
        async def page_index() -> None:  # noqa: C901, PLR0915
            """Index page of bucket module."""
            with frame("Manage Cloud Bucket on Aignostics Platform", left_sidebar=False):
                # Nothing to do here, just to show the page
                pass

            with ui.row(align_items="start").classes("w-full"):
                ui.markdown("""
                        ## Managing your cloud bucket
                        1. For analysis whole slide images are
                            temporarily stored in a cloud bucket of the Aignostics Platform.
                        3. The bucket is private and only accessible to you and restricted staff of Aignostics.
                        2. The bucket is securely hosted on Google Cloud in EU.
                        3. All data is encrypted in transit and at rest.
                        4. Any data is automatically deleted after 30 days.
                        5. You can manually delete data at any time using the form below.
                        """).classes("w-3/5")
                ui.space()
                ui.image("/bucket_assets/Google-Cloud-logo.png").classes("w-1/5").style("margin-top:1.25rem")

            async def _get_rows() -> list[dict[str, str]]:
                if bucket_form.spinner is not None:
                    bucket_form.spinner.set_visibility(True)
                if bucket_form.grid is not None:
                    bucket_form.grid.set_visibility(False)
                if bucket_form.delete_button is not None:
                    bucket_form.delete_button.set_visibility(False)
                objs = await run.io_bound(
                    Service.find_static,
                    detail=True,
                )
                if bucket_form.spinner is not None:
                    bucket_form.spinner.set_visibility(False)
                if bucket_form.grid is not None:
                    bucket_form.grid.set_visibility(True)
                if bucket_form.delete_button is not None:
                    bucket_form.delete_button.set_visibility(True)
                return [
                    {
                        "key": obj["key"],  # type: ignore
                        "last_modified": obj["last_modified"].astimezone().strftime("%x %X %Z"),  # type: ignore
                        "size": f"{obj['size'] / (1024 * 1024 * 1024):.2f} GB",  # type: ignore
                    }
                    for obj in objs
                ]

            async def _delete_selected() -> None:
                """Delete selected objects."""
                if bucket_form.grid is None or bucket_form.delete_button is None:
                    return
                selected_rows = await bucket_form.grid.get_selected_rows()
                if not selected_rows or selected_rows == []:
                    ui.notify("No objects selected.", type="warning")
                    return
                ui.notify(f"Deleting {len(selected_rows)} objects ...", type="info")
                bucket_form.delete_button.props(add="loading")
                try:
                    await run.io_bound(
                        Service.delete_objects_static,
                        [row["key"] for row in selected_rows],
                    )
                except Exception as e:  # noqa: BLE001
                    ui.notify(f"Error deleting objects: {e}", color="red", type="warning")
                    bucket_form.delete_button.props(remove="loading")
                    return
                ui.notify(f"Deleted {len(selected_rows)} objects.", type="positive")
                bucket_form.delete_button.props(remove="loading")
                bucket_form.delete_button.set_text("Delete")
                bucket_form.delete_button.disable()
                bucket_form.grid.options["rowData"] = await _get_rows()
                bucket_form.grid.update()

            async def _handle_grid_selection_changed() -> None:
                if bucket_form.grid is None or bucket_form.delete_button is None:
                    return
                rows = await bucket_form.grid.get_selected_rows()
                if rows:
                    bucket_form.delete_button.set_text(f"Delete {len(rows)} objects")
                    bucket_form.delete_button.enable()
                else:
                    bucket_form.delete_button.set_text("Delete")
                    bucket_form.delete_button.disable()

            bucket_form.spinner = ui.spinner(size="lg").classes(
                "absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 z-10"
            )
            bucket_form.spinner.set_visibility(False)

            bucket_form.grid = (
                ui.aggrid({
                    "columnDefs": [
                        {
                            "object": "Key",
                            "field": "key",
                            "checkboxSelection": True,
                            "filter": "agTextColumnFilter",
                        },
                        {
                            "headerName": "Last modified",
                            "field": "last_modified",
                            "filter": "agTextColumnFilter",
                        },
                        {
                            "headerName": "Size",
                            "field": "size",
                        },
                    ],
                    "rowData": await _get_rows(),
                    "rowSelection": "multiple",
                    "enableCellTextSelection": "true",
                    "autoSizeStrategy": {
                        "type": "fitCellContents",
                        "defaultMinWidth": 10,
                    },
                    "domLayout": "normal",
                })
                .classes("ag-theme-balham-dark" if app.storage.general.get("dark_mode", False) else "ag-theme-balham")
                .classes("full-width")
                .style("height: 310px")
                .mark("GRID_BUCKET")
                .on("selectionChanged", _handle_grid_selection_changed)
            )

            bucket_form.delete_button = (
                ui.button(
                    "Delete",
                    icon="delete",
                    on_click=_delete_selected,
                )
                .mark("BUTTON_DELETE_OBJECTS")
                .props("color=red")
                .classes("w-1/5")
            )
            bucket_form.delete_button.disable()

            ui.timer(
                interval=1,
                callback=lambda: bucket_form.grid.classes(
                    add="ag-theme-balham-dark" if app.storage.general.get("dark_mode", False) else "ag-theme-balham",
                    remove="ag-theme-balham" if app.storage.general.get("dark_mode", False) else "ag-theme-balham-dark",
                )
                if bucket_form.grid
                else None,
            )



"""Filesystem utilities."""

import platform
from pathlib import Path, PureWindowsPath

import appdirs
from showinfm.showinfm import show_in_file_manager

from ._constants import __is_running_in_read_only_environment__, __project_name__
from ._log import get_logger

# Constants
_WIN_DRIVE_MIN_LENGTH = 2  # Minimum length for a Windows drive path (e.g. C:)

logger = get_logger(__name__)


def sanitize_path_component(component: str) -> str:
    """
    Sanitize a single path component (e.g., filename, directory name).

    Rules:
    1. Replaces all colons with underscores (no drive letter exceptions).

    This function is useful for sanitizing individual path components like filepath.stem
    where you don't want drive letter special handling.

    Args:
        component (str): The path component to sanitize.

    Returns:
        str: The sanitized path component.
    """
    return component.replace(":", "_")


def sanitize_path(path: str | Path) -> str | Path:
    """
    Sanitize a filesystem path.

    Rules:
    1. If a Path is provided a Path will returned, otherwise a string will be returned.
    2. Colons will be replaced with underscores except if it's a Windows drive letter.
    3. On Windows: If the sanitized path is reserved a ValueError will be raised.

    Args:
        path (str | Path): The path to sanitize.
        replace_colon_with_underscore (bool): if True (default) will apply rule #2

    Returns:
        str | Path: The sanitized path.

    Raises:
        ValueError: If the sanitized path is reserved on Windows.
    """
    is_path_object = isinstance(path, Path)
    path_str = str(path)

    # Replace colons with underscores, except for Windows drive letters (e.g., C:/).
    # See https://stackoverflow.com/questions/25774337/colon-in-file-names-in-python
    # on NTFS creating hidden "streams" otherwise
    # MacOS replaces a colon with a / in Finder, which is confusing, see
    # https://superuser.com/questions/1797432/does-mac-os-accept-colon-in-filename
    if len(path_str) >= _WIN_DRIVE_MIN_LENGTH and path_str[1] == ":" and path_str[0].isalpha():
        # Windows drive letter case - preserve drive letter, sanitize the rest
        drive_part = path_str[0:2]
        remaining_part = sanitize_path_component(path_str[2:])
        path_str = drive_part + remaining_part
    else:
        # Regular case - sanitize the entire path as a component
        path_str = sanitize_path_component(path_str)

    if platform.system() == "Windows" and PureWindowsPath(path_str).is_reserved():
        message = f"The path '{path_str}' is reserved on Windows and cannot be used. Please choose a different path."
        raise ValueError(message)

    # Return the same type as input
    if is_path_object:
        return Path(path_str)
    return path_str


def get_user_data_directory(scope: str | None = None) -> Path:
    """Get the data directory for the service. Directory created if it does not exist.

    Args:
        scope (str | None): Optional scope for the data directory.

    Returns:
        Path: The data directory path.
    """
    directory = Path(appdirs.user_data_dir(__project_name__))
    if scope:
        directory /= scope
    if not __is_running_in_read_only_environment__:
        directory.mkdir(parents=True, exist_ok=True)
    return directory


def open_user_data_directory(scope: str | None = None) -> Path:
    """Open the user data directory in the file manager of the respective system platform.

    Args:
        scope (str | None): Optional scope for the data directory.

    Returns:
        Path: The data directory path.
    """
    directory = get_user_data_directory(scope)

    try:
        show_in_file_manager(str(directory.resolve()))
    except (OSError, RuntimeError, FileNotFoundError) as error:
        logger.warning(
            "Failed to open user data directory in file manager: %s. Directory path: %s",
            str(error),
            str(directory),
        )
    except Exception as error:  # noqa: BLE001
        # Catch any other unexpected exceptions to ensure function still returns directory path
        logger.warning(
            "Unexpected error opening user data directory in file manager: %s. Directory path: %s",
            str(error),
            str(directory),
        )

    return directory




"""
    Aignostics Platform API reference

     Pagination is done via `page` and `page_size`. Sorting via `sort` query parameter. The `sort` query parameter can be provided multiple times. The sorting direction can be indicated via `+` (ascending) or `-` (descending) (e.g. `/v1/applications?sort=+name)`.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from aignx.codegen.models.item_status import ItemStatus
from aignx.codegen.models.output_artifact_result_read_response import OutputArtifactResultReadResponse
from typing import Optional, Set
from typing_extensions import Self

class ItemResultReadResponse(BaseModel):
    """
    ItemResultReadResponse
    """ # noqa: E501
    item_id: StrictStr = Field(description="Item UUID generated by the Platform")
    application_run_id: StrictStr = Field(description="Application run UUID to which the item belongs")
    reference: StrictStr = Field(description="The reference of the item from the user payload")
    status: ItemStatus = Field(description=" When the item is not processed yet, the status is set to `pending`.  When the item is successfully finished, status is set to `succeeded`, and the processing results become available for download in `output_artifacts` field.  When the item processing is failed because the provided item is invalid, the status is set to `error_user`. When the item processing failed because of the error in the model or platform, the status is set to `error_system`. When the application_run is canceled, the status of all pending items is set to either `cancelled_user` or `cancelled_system`.     ")
    error: Optional[StrictStr]
    output_artifacts: List[OutputArtifactResultReadResponse] = Field(description=" The list of the results generated by the application algorithm. The number of files and their types depend on the particular application version, call `/v1/versions/{version_id}` to get the details.     ")
    __properties: ClassVar[List[str]] = ["item_id", "application_run_id", "reference", "status", "error", "output_artifacts"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ItemResultReadResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in output_artifacts (list)
        _items = []
        if self.output_artifacts:
            for _item_output_artifacts in self.output_artifacts:
                if _item_output_artifacts:
                    _items.append(_item_output_artifacts.to_dict())
            _dict['output_artifacts'] = _items
        # set to None if error (nullable) is None
        # and model_fields_set contains the field
        if self.error is None and "error" in self.model_fields_set:
            _dict['error'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ItemResultReadResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "item_id": obj.get("item_id"),
            "application_run_id": obj.get("application_run_id"),
            "reference": obj.get("reference"),
            "status": obj.get("status"),
            "error": obj.get("error"),
            "output_artifacts": [OutputArtifactResultReadResponse.from_dict(_item) for _item in obj["output_artifacts"]] if obj.get("output_artifacts") is not None else None
        })
        return _obj



"""System service."""

import json
import os
import platform
import re
import sys
import typing as t
from http import HTTPStatus
from pathlib import Path
from socket import AF_INET, SOCK_DGRAM, socket
from typing import Any, NotRequired, TypedDict
from urllib.request import getproxies

from dotenv import set_key as dotenv_set_key
from dotenv import unset_key as dotenv_unset_key
from pydantic_settings import BaseSettings
from requests import get

from ..utils import (  # noqa: TID252
    UNHIDE_SENSITIVE_INFO,
    BaseService,
    Health,
    __env__,
    __env_file__,
    __project_name__,
    __project_path__,
    __repository_url__,
    __version__,
    get_logger,
    get_process_info,
    load_settings,
    locate_subclasses,
)
from ._exceptions import OpenAPISchemaError
from ._settings import Settings

logger = get_logger(__name__)

JsonValue: t.TypeAlias = str | int | float | list["JsonValue"] | t.Mapping[str, "JsonValue"] | None
JsonType: t.TypeAlias = list[JsonValue] | t.Mapping[str, JsonValue]

# Note: There is multiple measurements and network calls
MEASURE_INTERVAL_SECONDS = 2
NETWORK_TIMEOUT = 5
IPIFY_URL = "https://api.ipify.org"


class RuntimeDict(TypedDict, total=False):
    """Type for runtime information dictionary."""

    environment: str
    username: str
    process: dict[str, Any]
    host: dict[str, Any]
    python: dict[str, Any]
    environ: dict[str, str]


class InfoDict(TypedDict, total=False):
    """Type for the info dictionary."""

    package: dict[str, Any]
    runtime: RuntimeDict
    settings: dict[str, Any]
    __extra__: NotRequired[dict[str, Any]]


class Service(BaseService):
    """System service."""

    _settings: Settings

    def __init__(self) -> None:
        """Initialize service."""
        super().__init__(Settings)

    @staticmethod
    def _is_healthy() -> bool:
        """Check if the service itself is healthy.

        Returns:
            bool: True if the service is healthy, False otherwise.
        """
        return True

    @staticmethod
    def _determine_network_health() -> Health:
        """Determine we can reach a well known and secure endpoint.

        - Checks if health endpoint is reachable and returns 200 OK
        - Uses requests library for a direct connection check without authentication

        Returns:
            Health: The healthiness of the network connection via basic unauthenticated request.
        """
        try:
            response = get(
                url=IPIFY_URL,
                headers={"User-Agent": f"aignostics-python-sdk/{__version__}"},
                timeout=NETWORK_TIMEOUT,
            )

            if response.status_code != HTTPStatus.OK:
                logger.error("'%s' returned '%s'", IPIFY_URL, response.status_code)
                return Health(
                    status=Health.Code.DOWN,
                    reason=f"'{IPIFY_URL}' returned status '{response.status_code}'",
                )
        except Exception as e:
            message = f"Issue reaching {IPIFY_URL}: {e}"
            logger.exception(message)
            return Health(status=Health.Code.DOWN, reason=message)

        return Health(status=Health.Code.UP)

    @staticmethod
    def health_static() -> Health:
        """Determine health of the system.

        - This method is static and does not require an instance of the service.
        - It is used to determine the health of the system without needing to pass the service.

        Returns:
            Health: The health of the system.
        """
        return Service().health()

    def health(self) -> Health:
        """Determine aggregate health of the system.

        - Health exposed by implementations of BaseService in other
            modules is automatically included into the health tree.
        - See utils/_health.py:Health for an explanation of the health tree.

        Returns:
            Health: The aggregate health of the system.
        """
        components: dict[str, Health] = {}
        for service_class in locate_subclasses(BaseService):
            if service_class is not Service:
                components[f"{service_class.__module__}.{service_class.__name__}"] = service_class().health()
        components["network"] = self._determine_network_health()

        # Set the system health status based on is_healthy attribute
        status = Health.Code.UP if self._is_healthy() else Health.Code.DOWN
        reason = None if self._is_healthy() else "System marked as unhealthy"
        return Health(status=status, components=components, reason=reason)

    def is_token_valid(self, token: str) -> bool:
        """Check if the presented token is valid.

        Returns:
            bool: True if the token is valid, False otherwise.
        """
        logger.info(token)
        if not self._settings.token:
            logger.warning("Token is not set in settings.")
            return False
        return token == self._settings.token.get_secret_value()

    @staticmethod
    def _get_public_ipv4(timeout: int = NETWORK_TIMEOUT) -> str | None:
        """Get the public IPv4 address of the system.

        Args:
            timeout (int): Timeout for the request in seconds.

        Returns:
            str: The public IPv4 address.
        """
        try:
            response = get(url=IPIFY_URL, timeout=timeout)
            response.raise_for_status()
            return response.text
        except Exception as e:
            message = f"Failed to get public IP: {e}"
            logger.exception(message)
            return None

    @staticmethod
    def _get_local_ipv4() -> str | None:
        """Get the local IPv4 address of the system.

        Returns:
            str: The local IPv4 address.
        """
        try:
            with socket(AF_INET, SOCK_DGRAM) as connection:
                connection.connect((".".join(str(1) for _ in range(4)), 53))
                return str(connection.getsockname()[0])
        except Exception as e:
            message = f"Failed to get local IP: {e}"
            logger.exception(message)
            return None

    @staticmethod
    def _is_secret_key(key: str) -> bool:
        """Determine if a key name indicates it contains secret information.

        This function uses two different matching strategies:
        1. Word boundary matching for terms like "id" and "auth" to avoid false positives
        2. Simple string matching for unambiguous secret terms like "token", "key", "secret", "password"

        Args:
            key: The key name to check

        Returns:
            bool: True if the key likely contains secret information
        """
        key_lower = key.lower()

        # Terms that require word boundary matching to avoid false positives
        # (e.g., "id" shouldn't match "valid" or "middle")
        word_boundary_terms = ["id"]

        # Terms that can use simple string matching as they're unambiguous
        string_match_terms = [
            "auth",
            "bearer",
            "cert",
            "credential",
            "hash",
            "jwt",
            "key",
            "nonce",
            "oauth",
            "password",
            "private",
            "salt",
            "secret",
            "seed",
            "session",
            "signature",
            "token",
        ]

        # Check word boundary terms using regex
        for term in word_boundary_terms:
            # Use regex to match word boundaries (non-alphanumeric characters)
            pattern = rf"(?:^|[^a-zA-Z]){re.escape(term)}(?:[^a-zA-Z]|$)"
            if re.search(pattern, key_lower):
                return True  # Check simple string match terms
        return any(term in key_lower for term in string_match_terms)

    @staticmethod
    def info(include_environ: bool = False, mask_secrets: bool = True) -> dict[str, Any]:  # type: ignore[override]
        """
        Get info about configuration of service.

        - Runtime information is automatically compiled.
        - Settings are automatically aggregated from all implementations of
            Pydantic BaseSettings in this package.
        - Info exposed by implementations of BaseService in other modules is
            automatically included into the info dict.

        Args:
            include_environ (bool): Whether to include environment variables in the info.
            mask_secrets (bool): Whether to mask information in environment variables identified as secrets

        Returns:
            dict[str, Any]: Service configuration.
        """
        import psutil  # noqa: PLC0415
        from uptime import boottime, uptime  # noqa: PLC0415

        bootdatetime = boottime()
        vmem = psutil.virtual_memory()
        swap = psutil.swap_memory()
        cpu_percent = psutil.cpu_percent(interval=MEASURE_INTERVAL_SECONDS)
        cpu_times_percent = psutil.cpu_times_percent(interval=MEASURE_INTERVAL_SECONDS)
        cpu_freq = None
        try:
            cpu_freq = psutil.cpu_freq()
        except RuntimeError:
            logger.warning("Failed to get CPU frequency.")  # Happens on macOS VM on GHA

        rtn: InfoDict = {
            "package": {
                "version": __version__,
                "name": __project_name__,
                "repository": __repository_url__,
                "local": __project_path__,
            },
            "runtime": {
                "environment": __env__,
                "username": psutil.Process().username(),
                "process": {
                    "command_line": " ".join(sys.argv),
                    "entry_point": sys.argv[0] if sys.argv else None,
                    "process_info": json.loads(get_process_info().model_dump_json()),
                },
                "host": {
                    "os": {
                        "platform": platform.platform(),
                        "system": platform.system(),
                        "release": platform.release(),
                        "version": platform.version(),
                    },
                    "machine": {
                        "cpu": {
                            "percent": cpu_percent,
                            "load_avg": psutil.getloadavg(),
                            "user": cpu_times_percent.user,
                            "system": cpu_times_percent.system,
                            "idle": cpu_times_percent.idle,
                            "arch": platform.machine(),
                            "processor": platform.processor(),
                            "count": os.cpu_count(),
                            "frequency": {
                                "current": cpu_freq.current if cpu_freq else None,
                                "min": cpu_freq.min if cpu_freq else None,
                                "max": cpu_freq.max if cpu_freq else None,
                            },
                        },
                        "memory": {
                            "percent": vmem.percent,
                            "total": vmem.total,
                            "available": vmem.available,
                            "used": vmem.used,
                            "free": vmem.free,
                        },
                        "swap": {
                            "percent": swap.percent,
                            "total": swap.total,
                            "used": swap.used,
                            "free": swap.free,
                        },
                    },
                    "network": {
                        "hostname": platform.node(),
                        "local_ipv4": Service._get_local_ipv4(),
                        "public_ipv4": Service._get_public_ipv4(),
                        "proxies": getproxies(),
                        "requests_ca_bundle": os.getenv("REQUESTS_CA_BUNDLE"),
                    },
                    "uptime": {
                        "seconds": uptime(),
                        "boottime": bootdatetime.isoformat() if bootdatetime else None,
                    },
                },
                "python": {
                    "version": platform.python_version(),
                    "compiler": platform.python_compiler(),
                    "implementation": platform.python_implementation(),
                    "sys.path": sys.path,
                    "interpreter_path": sys.executable,
                },
            },
            "settings": {},
        }

        runtime = rtn["runtime"]
        if include_environ:
            if mask_secrets:
                runtime["environ"] = {
                    k: "*********" if Service._is_secret_key(k) else v for k, v in sorted(os.environ.items())
                }
            else:
                runtime["environ"] = dict(sorted(os.environ.items()))

        settings: dict[str, Any] = {}
        for settings_class in locate_subclasses(BaseSettings):
            settings_instance = load_settings(settings_class)
            env_prefix = settings_instance.model_config.get("env_prefix", "")
            settings_dict = json.loads(
                settings_instance.model_dump_json(context={UNHIDE_SENSITIVE_INFO: not mask_secrets})
            )
            for key, value in settings_dict.items():
                flat_key = f"{env_prefix}{key}".upper()
                settings[flat_key] = value
        rtn["settings"] = {k: settings[k] for k in sorted(settings)}

        # Convert the TypedDict to a regular dict before adding dynamic service keys
        result_dict: dict[str, Any] = dict(rtn)

        for service_class in locate_subclasses(BaseService):
            if service_class is not Service:
                service = service_class()
                result_dict[service.key()] = service.info(mask_secrets=mask_secrets)

        logger.info("Service info: %s", result_dict)
        return result_dict

    @staticmethod
    def openapi_schema() -> JsonType:
        """
        Get OpenAPI schema of the webservice API provided by the platform.

        Returns:
            dict[str, object]: OpenAPI schema.

        Raises:
            OpenAPISchemaError: If the OpenAPI schema file cannot be found or is not valid JSON.
        """
        schema_path = Path(__file__).parent.parent.parent.parent / "codegen" / "in" / "api.json"
        try:
            with schema_path.open(encoding="utf-8") as f:
                return json.load(f)  # type: ignore[no-any-return]
        except (FileNotFoundError, json.JSONDecodeError) as e:
            raise OpenAPISchemaError(e) from e

    @staticmethod
    def _get_env_files_paths() -> list[Path]:
        """Get the paths of the environment files.

        Returns:
            list[Path]: List of environment file paths.
        """
        return __env_file__

    @staticmethod
    def dotenv_get(key: str) -> str | None:
        """Get value of key in environment.

        Args:
            key (str): The key to add.

        Returns:
            str | None: The value of the key if it exists, None otherwise.
        """
        return os.getenv(key, None)

    @staticmethod
    def dotenv_set(key: str, value: str) -> None:
        """Set key-value pair in primary .env file, unset in alternative .env files.

        Args:
            key (str): The key to add.
            value (str): The value to add.

        Raises:
            ValueError: If the primary .env file does not exist.

        """
        Service.dotenv_unset(key)

        dotenv_path = Service._get_env_files_paths()[0]  # Primary .env file
        if not dotenv_path.is_file():
            message = f"Primary .env file '{dotenv_path!s}' does not exist, canceling update of .env files"
            logger.error(message)
            raise ValueError(message)

        dotenv_set_key(dotenv_path=str(dotenv_path.resolve()), key_to_set=key, value_to_set=value, quote_mode="auto")
        os.environ[key] = value

    @staticmethod
    def dotenv_unset(key: str) -> int:
        """Unset key-value pair in all .env files.

        Args:
            key (str): The key to remove.

        Returns:
            int: The number of times the key has been removed across files.
        """
        removed_count = 0
        for dotenv_path in Service._get_env_files_paths():
            if not dotenv_path.is_file():
                message = f"File '{dotenv_path!s}' does not exist, skipping update"
                logger.debug(message)
                continue
            dotenv_unset_key(dotenv_path=str(dotenv_path.resolve()), key_to_unset=key, quote_mode="auto")
        os.environ.pop(key, None)
        return removed_count

    @staticmethod
    def remote_diagnostics_enabled() -> bool:
        """Check if remote diagnostics are enabled.

        Returns:
            bool: True if remote diagnostics are enabled, False otherwise.
        """
        return (
            Service.dotenv_get(f"{__project_name__.upper()}_SENTRY_ENABLED") == "1"
            and Service.dotenv_get(f"{__project_name__.upper()}_LOGFIRE_ENABLED") == "1"
        )

    @staticmethod
    def remote_diagnostics_enable() -> None:
        """Enable remote diagnostics via Sentry and Logfire. Data stored in EU data centers.

        Raises:
            ValueError: If the environment variable cannot be set.
        """
        Service.dotenv_set(f"{__project_name__.upper()}_SENTRY_ENABLED", "1")
        Service.dotenv_set(f"{__project_name__.upper()}_LOGFIRE_ENABLED", "1")

    @staticmethod
    def remote_diagnostics_disable() -> None:
        """Disable remote diagnostics."""
        Service.dotenv_unset(f"{__project_name__.upper()}_SENTRY_ENABLED")
        Service.dotenv_unset(f"{__project_name__.upper()}_LOGFIRE_ENABLED")

    @staticmethod
    def http_proxy_enable(
        host: str,
        port: int,
        scheme: str,
        ssl_cert_file: str | None = None,
        no_ssl_verify: bool = False,
    ) -> None:
        """Enable HTTP proxy.

        Args:
            host (str): The host of the proxy server.
            port (int): The port of the proxy server.
            scheme (str): The scheme of the proxy server (e.g., "http", "https").
            ssl_cert_file (str | None): Path to the SSL certificate file, if any.
            no_ssl_verify (bool): Whether to disable SSL verification

        Raises:
            ValueError: If both 'ssl_cert_file' and 'ssl_disable_verify' are set.
        """
        url = f"{scheme}://{host}:{port}"
        Service.dotenv_set("HTTP_PROXY", url)
        Service.dotenv_set("HTTPS_PROXY", url)
        if ssl_cert_file is not None and no_ssl_verify:
            message = "Cannot set both 'ssl_cert_file' and 'ssl_disable_verify'. Please choose one."
            logger.warning(message)
            raise ValueError(message)
        if no_ssl_verify:
            Service.dotenv_set("SSL_NO_VERIFY", "1")
            Service.dotenv_set("SSL_CERT_FILE", "")
            Service.dotenv_set("REQUESTS_CA_BUNDLE", "")
            Service.dotenv_set("CURL_CA_BUNDLE", "")
        else:
            Service.dotenv_unset("SSL_NO_VERIFY")
            Service.dotenv_unset("SSL_CERT_FILE")
            Service.dotenv_unset("REQUESTS_CA_BUNDLE")
            Service.dotenv_unset("CURL_CA_BUNDLE")
            if ssl_cert_file:
                file = Path(ssl_cert_file).resolve()
                if not file.is_file():
                    message = f"SSL certificate file '{ssl_cert_file}' does not exist."
                    logger.warning(message)
                    raise ValueError(message)
                Service.dotenv_set("SSL_CERT_FILE", str(ssl_cert_file))
                Service.dotenv_set("REQUESTS_CA_BUNDLE", str(ssl_cert_file))
                Service.dotenv_set("CURL_CA_BUNDLE", str(ssl_cert_file))

    @staticmethod
    def http_proxy_disable() -> None:
        """Disable HTTP proxy."""
        Service.dotenv_unset("HTTP_PROXY")
        Service.dotenv_unset("HTTPS_PROXY")
        Service.dotenv_unset("SSL_CERT_FILE")
        Service.dotenv_unset("SSL_NO_VERIFY")
        Service.dotenv_unset("REQUESTS_CA_BUNDLE")
        Service.dotenv_unset("CURL_CA_BUNDLE")
